@@ -4,6 +4,11 @@
 from .test_base import *
 from .test_containers import *
 from .test_declarations import *
+from .test_deprecation import *
+from .test_django import *
 from .test_fuzzy import *
+from .test_helpers import *
 from .test_using import *
 from .test_utils import *
+from .test_alchemy import *
+from .test_mongoengine import *
