@@ -23,7 +23,7 @@
 
 import factory
 
-from cyclic import bar
+from . import bar as bar_mod
 
 class Foo(object):
     def __init__(self, bar, x):
@@ -32,7 +32,8 @@ class Foo(object):
 
 
 class FooFactory(factory.Factory):
-    FACTORY_FOR = Foo
+    class Meta:
+        model = Foo
 
     x = 42
-    bar = factory.SubFactory(bar.BarFactory)
+    bar = factory.SubFactory(bar_mod.BarFactory)
